/**
 * Title:        DeviceFinder.java<p>
 * Description:  Allows you to find all video device that support yuv format and audio devices that suppor linear format on the system<p>
 * Copyright:    Copyright (c) <p>
 * Company:      George's Company<p>
 * @author George Voulgarakis
 * @version $Id: DeviceFinder.java,v 1.1 2000/11/28 03:41:24 konrad Exp $DeviceFinder.java,v 1.1 2000/11/11 1:00:00am George Exp $
 */

package media.config;
import java.util.*;
import javax.media.*; 
import javax.media.Format.*;
import javax.media.format.*;

public class DeviceFinder{

   private Vector videoDevices = new Vector();    //List of Video Devices 
   private Vector audioDevices = new Vector();    //List of Audio Devices
   private static final String videoFormat = "YUV";      //Video format used
   private static final String audioFormat = "LINEAR";   //Audio format used

   /**
    * Creates a new Device Finder Object
    */
   public DeviceFinder(){

     this.findVideoDevices();  //Gets VideoDevices
     this.findAudioDevices();  //Gets AudioDevices

   }

   /**
    *  Finds audio devices with linear format
    */

   private void findAudioDevices(){
      CaptureDeviceInfo[] AUDIO = null; 
      audioDevices = CaptureDeviceManager.getDeviceList(new AudioFormat(audioFormat));
   }


   /**
    * Finds video devices with yuv format
    */

   private void findVideoDevices(){

      CaptureDeviceInfo[] VIDEO;      
      videoDevices = CaptureDeviceManager.getDeviceList(new VideoFormat(videoFormat));
     
   }

   /**
    *  Gets the VideoDeviceNames as String Array
    */
   public String[] getVideoDeviceNames() {

      int num_video_drivers = videoDevices.size();  //number of video devices
      String[] vDeviceNames = new String[num_video_drivers + 1];  //video device names
      Iterator vd = videoDevices.iterator();
      vDeviceNames[0] = "file"; //puts a file as the first device
      int counter = 1;
      while (vd.hasNext()){
         vDeviceNames[counter] = ((CaptureDeviceInfo) (vd.next())).getName(); //gets name of video device
       counter++;
      }
      return vDeviceNames;
   }

   /**
    *  Gets the VideoDeviceLocators as a String Array
    */
   public String[] getVideoDeviceLocators() {

      int num_video_drivers = videoDevices.size();  //number of video devices
      String[] vDeviceLocators = new String[num_video_drivers + 1];  //video device locators
      Iterator vd = videoDevices.iterator();
      vDeviceLocators[0] = "file"; //puts a file as the first device
      int counter = 1;
      while (vd.hasNext()){
         vDeviceLocators[counter] = ((CaptureDeviceInfo) (vd.next())).getLocator().toExternalForm(); //gets locators of video devices
       counter++;
      }
      return vDeviceLocators;
   }

   /**
    * Gets the AudioDeviceNames as a String Array
    */

   public String[] getAudioDeviceNames() {

      int num_audio_drivers = audioDevices.size();                //number of audio devices
      String[] aDeviceNames = new String[num_audio_drivers + 1];  //audio device names
      Iterator ad = audioDevices.iterator();
      aDeviceNames[0] = "file"; //puts a file as the first device
      int counter = 1;

      while (ad.hasNext()){
	  aDeviceNames[counter] = ((CaptureDeviceInfo) (ad.next())).getName(); //gets name of audio device
        counter++;
      }
      return aDeviceNames;
   }

    /**
    * Gets the AudioDeviceLocators as a String Array
    */

   public String[] getAudioDeviceLocators() {

      int num_audio_drivers = audioDevices.size();                   //number of audio devices
      String[] aDeviceLocators = new String[num_audio_drivers + 1];  //audio device names
      Iterator ad = audioDevices.iterator();
      aDeviceLocators[0] = "file"; //puts a file as the first device
      int counter = 1;
      while (ad.hasNext()){
        aDeviceLocators[counter] = ((CaptureDeviceInfo) (ad.next())).getLocator().toExternalForm(); //gets locators of audio device
        counter++;
      }
      return aDeviceLocators;
   }

   /**
    *  Gets locator of Audio Device given the locator
    */

   public String getAudioLocator(String name) {

     CaptureDeviceInfo device = new CaptureDeviceInfo();              //A capture device
     boolean found = false;                                           //boolean if the string is found
     Iterator ad = audioDevices.iterator();                           //An iterator to the audio devices
  
     if(audioDevices.size() > 0){
       while(ad.hasNext() && !(found)){
         device = (CaptureDeviceInfo) (ad.next());
         if(device.getName().equals(name)){
            found = true;
         }
       }     

       return device.getLocator().toExternalForm();
     }
     else
        return "";

   }
   
   /**
    *  Gets locator of Video Device given the name
    */

   public String getVideoLocator(String name) {
  
     CaptureDeviceInfo device = new CaptureDeviceInfo();              //A capture device
     boolean found = false;                                           //boolean if the string is found
     Iterator vd = videoDevices.iterator();                           //An iterator to the video devices
  
     if(videoDevices.size() > 0){
       while(vd.hasNext() && !(found)){
         device = (CaptureDeviceInfo) (vd.next());
         if(device.getName().equals(name)){
            found = true;
         }
       }     

       return device.getLocator().toExternalForm();
     }
     else
        return "";
   }

   /**
    *  Gets name of Audio Device given the locator
    */

   public String getAudioName(String locator) {

     CaptureDeviceInfo device = new CaptureDeviceInfo();               //A capture device
     boolean found = false;                                            //boolean if the string is found
     Iterator ad = audioDevices.iterator();                            //An iterator to the audio devices

     if(audioDevices.size() > 0){
       while(ad.hasNext() && !(found)){
         device = (CaptureDeviceInfo) (ad.next());
         if(device.getLocator().toExternalForm().equals(locator)){
            found = true;
         }
       }     

       return device.getName();
     }
     else
        return "";

   }
  
   /**
    *  Gets name of Video Device given the locator
    */
   
   public String getVideoName(String locator) {

     CaptureDeviceInfo device = new CaptureDeviceInfo();               //A capture device
     boolean found = false;                                            //boolean if the string is found
     Iterator vd = videoDevices.iterator();                            //An iterator to the video devices

     if(videoDevices.size() > 0){
       while(vd.hasNext() && !(found)){
         device = (CaptureDeviceInfo) (vd.next());
         if(device.getLocator().toExternalForm().equals(locator)){
            found = true;
         }
       }     

       return device.getName();
     }
     else
        return "";
     
   }

   /**
    *  Requeries for video and audio devices
    */

   public void requeryDevices() {

      videoDevices = new Vector(); //clears video devices 
      audioDevices = new Vector(); //clears audio devices
      this.findVideoDevices();  //find Video devices
      this.findAudioDevices();  //find audio devices
   }
 }








