package media.network;
import java.util.*;
import javax.swing.ImageIcon;
import media.config.Config;
import media.common.*;
import media.gui.*;
import java.net.*;
import java.io.*;
/**
 * Title:        Network.java<br>
 * Description:  Handles the TCP, RTP and Jini <br>
 * Copyright:    Konrad Rzeszutek<br>
 * Company:      darnok.dhs.org<br>
 *
 * Version:      $Id: Network.java,v 1.2 2000/11/16 05:53:53 konrad Exp $ <br>
 *
 * Setups up the TCP connection, handles Jini services.
 * Also handles RTP connections.
 */

public class Network {

    /* Our class references */
    public  Config config                  = null;
    public  AudioReceiver audio_receiver     = null;
    public  VideoReceiver video_receiver     = null;
    public  VideoSender video_sender         = null;
    public  AudioSender audio_sender         = null;
    private User me = null;
    private User you = null;
    private PingThread ping = null;

    /* these values are returned when 'connect','disconnect' is invoked */
    public final int OK                       = 123;
    public final int BUSY                     = 432;
    public final int DEAD                     = 1323;
    public final int ERROR                    = 32;
    private Users hostList = null;
    private Socket theSocket = null;
    private ObjectInputStream input = null;
    private ObjectOutputStream output = null;
    public  OcularBase gui = null;

    public Network (OcularBase gui, Config config, AudioReceiver audio, VideoReceiver video, AudioSender audio_sender, VideoSender video_sender) {
	this.gui = gui;
	this.config = config;
	this.audio_receiver = audio;
	this.video_receiver = video;

        this.audio_sender = audio_sender;
        this.video_sender = video_sender;

	hostList = new Users( );
	TCPServer server = new TCPServer(this);
	server.start( );
	config.write("Network started");
    }

    public Users getHostList( ){
	return hostList;
    }
    public void addUser(User user){
	hostList.addNewHost(user);
    }
    public void removeUser(User user){
	hostList.removeHost(user);
    }

    public void declareDead( ){
	config.error("Somebody died!");
	me.updateStatus("AVAILABLE");
	you.updateStatus("DEAD");
	disconnect();
	gui.resetConnected();
    }

    public Config giveConfig( ){
	return config;
    }
    public void closeStreams( ){

	try{
	    if(ping != null){
		ping.join( );
		config.error("Ping thread has died and joined");
	    }

	    input.close( );
	    output.close( );
	    theSocket.close( );
	    input = null;
	    output = null;
	    theSocket= null;
	}catch(Exception e){
	    config.error("Couldn't close: "+e.toString( ));
	}
    }//end method

    private void refreshStatus( ){
	if(hostList.size( )>2){
	    for(int i=0; i< hostList.size( );i++){
		User user =(User)hostList.get(i);	
		try{
		    if(!(user.getHostName( ).equals(InetAddress.getLocalHost( ).getHostName()))){
			Socket socket = new Socket(user.getHostName( ),8888);
			ObjectInputStream input = new ObjectInputStream(socket.getInputStream( ));
			ObjectOutputStream output = new ObjectOutputStream(socket.getOutputStream( ));
			output.flush( );		
			output.writeObject("STATUS");
			output.flush( );		
			output.writeObject(user.getName( ));
			output.flush( );		
			String status = (String)input.readObject( );		
			user.updateStatus(status);		
			input.close( );
			output.close( );
			socket.close( );		
		    }//end inner if
		}catch(Exception e){
		    config.error("Error while waiting for information from from the server"+e.toString( ));
		}//end catch
	    }//end for
	}//end outer if
    }//end method refreshStatus

    /**
     * Gets users (uses Jini)
     */
    public Object[][] getUsers( ){
	config.write("Refreshing hosts");
	refreshStatus( );
	config.write("Updating user list");
	Object[][] value = null;

	int records = hostList.size( );

	/* 3 is set b/c we have: 'status', 'name', and 'hostname' */
	value  = new Object[records][3];

	ImageIcon busy = config.SCHEME_OBJ.busy_icon;

	ImageIcon avail = config.SCHEME_OBJ.available_icon;

	ImageIcon dead = config.SCHEME_OBJ.na_icon;


	for (int i = 0; i < records; i++) {
	    User user = (User)hostList.get(i);
	    String status = user.getStatus( );
	    if(status.equals("AVAILABLE"))
		value[i][0] = avail;
	    else if(status.equals("BUSY") || (status.equals("WAITING")))
		value[i][0] = busy;
	    else
		value[i][0] = dead;

	    value[i][1] = user.getName( );
	    value[i][2]	= user.getHostName( );
	}
	return value;
    }


    /**
     * Establish that the user wants to speak with a dude at the particular
     * hostname, port, and username.
     *
     */

    public int connect(String hostname, String name) {
	config.write("Connecting with "+name+" @ "+hostname+" ");
	
	//User me=null;
	String localhost = null;
	boolean goOn = true;
	
	for(int i=0; i<hostList.size( );i++){
	    try{
		localhost = InetAddress.getLocalHost( ).getHostName( );
	    }catch(Exception e){
		config.error("Major flaw"+e.toString( ));
	    }
	    User user = (User) hostList.get(i);
	    if(user.getHostName( ).equalsIgnoreCase(localhost))
		me = user;	     
	    else if(user.getHostName( ).equalsIgnoreCase(hostname))
		you = user;
	}//end for

	try{
	    theSocket = new Socket(hostname,8888);
	}catch(IOException e){

	    config.error("Couldn't establish the socket connection");
	    goOn = false;
	}
	if (goOn) {
	    try{
		input = new ObjectInputStream(theSocket.getInputStream( ));
		output = new ObjectOutputStream(theSocket.getOutputStream( ));
		output.writeObject("CONNECT");
		output.flush( );
		output.writeObject(me.getHostName( ));
		output.flush( );
		String message = (String) input.readObject( );
		if(message.equalsIgnoreCase("WELCOME")){
		    me.updateStatus("BUSY");
		    you.updateStatus("BUSY");
		    
		    output.writeObject(me.getHostName( ));
		    output.flush( );
		    output.writeObject(new Integer(config.RECV_AUD_PORT));
		    output.flush( );
		    output.writeObject(new Integer(config.RECV_VID_PORT));
		    output.flush( );
		    config.error("RecvAudPort :"+config.RECV_AUD_PORT);
		    config.error("RecvVidPort :"+config.RECV_VID_PORT);
		    
		
		    String host = (String) input.readObject( );
		    InetAddress ip_addr = InetAddress.getByName(host);
		    String ip_host = new String(ip_addr.getHostAddress());
		    
		    // F	un part.
		    config.SEND_HOSTNAME = ip_host;
		    config.SEND_AUD_PORT = ((Integer) input.readObject( )).intValue( );
		    config.SEND_VID_PORT = ((Integer) input.readObject( )).intValue( );
		    config.error("SendAudPort :"+config.SEND_AUD_PORT);
		    config.error("SendVidPort :"+config.SEND_VID_PORT);
		    
		    InetAddress ip_addr_my = InetAddress.getLocalHost();
		    
		    config.RECV_HOSTNAME = ip_host;
		    config.error("Hostname "+config.SEND_HOSTNAME+","+config.RECV_HOSTNAME);
		    ping = new PingThread(this,input, output);
		    ping.start( );	
		}//end if welcome	
		else if(message.equalsIgnoreCase("WELCOME BACK")){
		
		    me.updateStatus("AVAILABLE");
		    you.updateStatus("AVAILABLE");
		    String host = (String) input.readObject( );
		    InetAddress ip_addr = InetAddress.getByName(host);
		    String ip_host = new String(ip_addr.getHostAddress());
		    
		    // F	un part.
		    config.SEND_HOSTNAME = ip_host;
		    config.SEND_AUD_PORT = ((Integer) input.readObject( )).intValue( );
		    config.SEND_VID_PORT = ((Integer) input.readObject( )).intValue( );
		    config.error("SendAudPort :"+config.SEND_AUD_PORT);
		    config.error("SendVidPort :"+config.SEND_VID_PORT);
		    
		    InetAddress ip_addr_my = InetAddress.getLocalHost();
		    
		    config.RECV_HOSTNAME = ip_host;
		    config.error("Hostname "+config.SEND_HOSTNAME);
		    config.error("Hostname(R) "+config.RECV_HOSTNAME);

		    ping = new PingThread(this,input, output);
		    ping.start( );
		}//end else if welcome	 back		
		else if(message.equalsIgnoreCase("BUZZ OFF")){
		    closeStreams( );
		    config.write("Session closed by network");
		    goOn = false;
		}//end else if buzz	
	   } catch(Exception e){
	       System.err.println(e.toString());
	       config.write("Error while retrieving the message from the server of host");
	       goOn = false;
	   }
	}

	// Killing all RTP
	config.write("Killing RTP Audio/Video");
	audio_receiver.resetAudio();
	video_receiver.resetVideo();
        audio_sender.resetAudio();
        video_sender.resetVideo();

	
	if (goOn)
	    config.write("Starting AV sender");
	
	if (goOn && video_sender.startVideoSender() != VideoSender.OK)
	    if (gui.error("Couldn't start Video transmitter!")) {
		closeSession(); goOn = false; }
	
	if (goOn && audio_sender.startAudioSender() != AudioSender.OK)
	    if (gui.error("Couldn't start Audio transmitter!")) {
		closeSession(); goOn =false;; }
	
	
	if (goOn)
	    config.write("Starting AV listeners");
	
	if (goOn && audio_receiver.startAudioReceiver() != AudioReceiver.OK)
	    if (gui.error("Couldn't start Audio RTP listener!")) {
		closeSession(); goOn = false; }
	if (goOn && video_receiver.startVideoReceiver() != VideoReceiver.OK)
	    if (gui.error("Couldn't start Video RTP listener!")) {
		closeSession(); goOn = false; }
	
	config.STATE = config.I_AM_AVAILABLE;
	if (goOn)
	    return OK;
	
	return ERROR;
    }
    
    public void keepAlive( ){
	me.updateStatus("AVAILABLE");
	you.updateStatus("AVAILABLE");
    }
    /**
     * Disconnects from the current user.
     *    
     */
    public int disconnect(){

	int returnState = OK;
	if (ping != null)
	    ping.changePing( );

	if (me != null)
	    me.updateStatus("AVAILABLE");
	if (you != null)
	    you.updateStatus("AVAILABLE");

	config.STATE = config.I_AM_AVAILABLE;
	
	// Drop RTP sessions (incoming).
	config.write("Killing RTP Audio/Video Listeners");
	if (audio_receiver.resetAudio() == AudioReceiver.ERROR)
	    returnState = ERROR;
 
	if (video_receiver.resetVideo() == VideoReceiver.ERROR)
	    returnState = ERROR;

	// Drop RTP sessions (outgoing)
	config.write("Killing RTP Audio/Video Senders");
        if (audio_sender.resetAudio() == AudioSender.ERROR)
	    returnState = ERROR;

        if (video_sender.resetVideo() == VideoSender.ERROR)
	    returnState = ERROR;

	
	config.write("Disconnected");
	return returnState;

    }
   /**
     * Kills the RTP and resets the GUI to NONE state.
     */
    public void closeSession() {

        this.disconnect();
        gui.resetConnected();
        gui.state = gui.NONE;
    }

    class PingThread extends Thread{

	private Network net = null;
	private boolean isPinging = false;
	private ObjectOutputStream output = null;
	private ObjectInputStream input = null;

	/**
	 * A simple Thread class that will take
	 * care of the <i> pings<\i> that this
	 * client sends to the corresponding 
	 * client that he is connected to
	 */

	public PingThread(Network net,ObjectInputStream in, ObjectOutputStream out){
	    this.net = net;
	    this.input = in;
	    this.output = out;	  
	    isPinging = true;	
	}//end constructor

	public void changePing( ){
	    isPinging = false;   
	    try{
		net.keepAlive( );
		output.writeObject("BYE");
		output.flush( );
		config.write("Bye Bye!");

	    }catch(Exception e){
		config.error("He will be really missed");

	    }
	}

	public void run( ){
	    while(isPinging){
		try{
		    this.sleep(config.INTERVAL);		    
		    receivePong( );
		    config.error("PONG!");
		    if(isPinging){
			sendPing( );
			config.error("PING!");
		    }
		  
		}catch(IOException e){
		    /**
		     *  Tell that the pong was
		     *  not received
		     */		 
		    net.declareDead();
		    net.closeStreams( );		

		}
		catch(Exception e){//tcp connection failed

		    config.error("Client walked away! ["+e.toString()+ "]");	
		    isPinging = false;
		    net.declareDead( );
		    net.closeStreams( );
		}//end catch
	    }//end while
	}//end run

	/**
	 *  Try to recieve the corresponding pong
	 */

	private void receivePong( ) throws IOException, ClassNotFoundException{
	    String in ="";	    
	    in = (String) input.readObject( );		
	    if(in.equalsIgnoreCase("PONG"))
		isPinging = true;	
	    else{ //the string is not ping	
		isPinging = false;
		throw new IOException("Pong not received");
	    }
	}//end receivePong

	/**
	 *  Send the ping message to the client
	 *  on the other side.
	 */

	private void sendPing( )throws IOException{
	    output.writeObject("PING");
	    output.flush( );
	}

    }//end PingThread class

}//end of Network class
